/*-----------------------------------------------------------------
  Released under the MIT License  - see
  http://www.opensource.org/licenses/mit-license.php

  travel_clock by Jack Botner

  interrupt.c

  This module contains all the interrupt handlers.

  20201212: program written
  20210526: move encoder I/O code out of interrupt handler
-----------------------------------------------------------------*/
#include <avr/io.h>
#include <avr/interrupt.h>
#include "main.h"
#include "led.h"

// Global variables

extern volatile uint8_t ucLed[];				// led digit values
extern volatile uint8_t ucDp[];					// which decimals are on
extern volatile uint8_t ucEventFlags;			// system event flags
extern volatile uint8_t ucLedTimeSliceLimit;    // controls the display brightness
extern volatile uint8_t ucAlarmState;           // alarm state
extern volatile uint16_t ucSpkrToggles;         // so alarm does not sound too long

static volatile uint8_t ucLedTimeSliceCtr=0;

volatile uint16_t	uiAdc=0;			// last adc sample

/*-----------------------------------------------------------------
  ISR( ADC_vect ) - ADC conversion complete interrupt handler

  ADC3 is monitoring the photocell.
-----------------------------------------------------------------*/
ISR( ADC_vect )
{
  // ADCL must be read before ADCH.
  uiAdc = (uint16_t) ADCL | ( (uint16_t) ADCH << 8 );
  set_bit( ucEventFlags, EVENT_ADC_COMPLETE );
}

/*-----------------------------------------------------------------
  ISR(TIMER0_OVF_vect)

  Timer 0 overflow interrupt handler - tests for button presses

  System clock frequency = 16 mHz
  System clock period = 62.5 ns
  Prescaler = 256
  Counts = 63
  Interrupt period = 1 ms
------------------------------------------------------------------*/
ISR(TIMER0_OVF_vect)
{
  set_bit( ucEventFlags, EVENT_1MS );    // for encoder I/O

  // If alarm active, toggle on and off to make sound
  if ( ucAlarmState == ALARM_ON_ACTIVE )
  {
    if ( test_bit( SPEAKER_PORT, SPEAKER_PIN ) )
    {
      ++ucSpkrToggles;
      clear_bit( SPEAKER_PORT, SPEAKER_PIN );	 // beeper off
    }
    else
      set_bit( SPEAKER_PORT, SPEAKER_PIN );		 // beeper on
  }

  TCNT0 = INIT_CTR0;
}

/*-----------------------------------------------------------------
  ISR(TIMER1_OVF_vect)

  Timer 1 overflow interrupt handler for 1/2 second events
  - beeper
  - blinking digits
  - time refresh

  System clock frequency = 16 mHz
  System clock period = 62.5 ns
  Prescaler = 256
  Counts = 31250
  Interrupt period = 500 ms
------------------------------------------------------------------*/
ISR(TIMER1_OVF_vect)
{
  set_bit( ucEventFlags, EVENT_HALFSEC );

  TCNT1 = INIT_CTR1;
}

/*-----------------------------------------------------------------
  ISR(TIMER2_OVF_vect)

  Timer 2 overflow interrupt handler - LED digit switching

  System clock frequency = 16 mHz
  System clock period = 62.5 ns
  Prescaler = 32
  Counts = 26 -> 52
  Interrupt period = 50us  -> 100us
------------------------------------------------------------------*/
ISR(TIMER2_OVF_vect)
{
  // We want to control the LED brightness in LED_TIME_SLICES / 4 steps.
  // So we consider each interrupt a time slice, and it
  // requires 4 time slices to illuminate all the LEDs.
  // The controlling factor is ucLedTimeSliceLimit, which must
  // be a value between 0 and LED_TIME_SLICES which is a multiple of 4.
  // (A value of 0 means the display will be blank; LED_TIME_SLICES
  // means max brightness.)
  // To do this the interrupt interval has been sped up
  // from 1 ms to 100 us.

  static volatile uint8_t ucIndex=0;	// digit selection index

  // Show the next digit on the LED display.
  ledBlankDisplay();
  clear_bit( COLON_PORT, COLON_PIN );

  if ( ucLedTimeSliceCtr < ucLedTimeSliceLimit )
  {
    ledOutputData( ( ucIndex + 1 ), ucLed[ucIndex], ucDp[ucIndex] );
    ledSelectDigit( ucIndex );
    set_bit( COLON_PORT, COLON_PIN );

    ++ucIndex;
    if ( ucIndex > 3 )
      ucIndex = 0;
  }
  
  ++ucLedTimeSliceCtr;
  if ( ucLedTimeSliceCtr == LED_TIME_SLICES )
    ucLedTimeSliceCtr = 0;

  TCNT2 = INIT_CTR2;
}
